
# q-expansion of newform 819.3.d.b, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 819
weight = 3
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[92, [1]], [703, [-1]], [379, [-1]]]
aps_data = [[0], [0], [3], [7], [0], [-13], [0], [25], [45], [33], [-55], [0], [-30], [-5], [-81], [-15], [90], [0], [0], [0], [29], [67], [159], [-165], [-131], [0], [0], [150], [0], [-135], [-110], [0], [0], [0], [0], [0], [0], [0], [-9], [0], [-267], [0], [-18], [0], [0], [0], [-397], [121], [-246], [-10], [-375], [0], [365], [0], [0], [-435], [0], [-290], [-265], [0], [0], [-261], [-439], [0], [0], [0], [0], [-145], [-330], [-355], [594], [0], [0], [-710], [0], [66], [678], [781], [0], [-755], [0], [0], [0], [0], [0], [-795], [0], [0], [-870], [0], [0], [615], [0], [-618], [0], [0], [-1005], [0], [0], [0], [275], [0], [0], [1137], [-1133], [-146], [-849], [1011], [1077], [0], [0], [0], [0], [1030], [0], [993], [-586], [0], [150], [957], [1205], [1255], [0], [0], [-815], [873], [0], [0], [0], [-1459], [0], [0], [-773], [695], [675], [485], [-246], [1], [0], [657], [-250], [0], [-1630], [0], [0], [1650], [1381], [0], [0], [0], [0], [0], [-1510], [0], [995], [453], [-1438], [-1845], [0], [1875], [0], [-1815], [0], [0], [0], [1791], [1618], [0]]
